%% font font_9x15 fixed_9x15
%% font font_10x20 fixed_10x20


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

/debug 0 def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%
% input event handling
%
% ( key ) ==> ( input_buffer menu_entry action )
%
% key
%  bit 0-7	ascii
%  bit 8-15	scan code
%  bit 16-32	status bits (ctrl, shift...)
%
% action
%    0:		ok, stay in input loop
%    1:		switch to text mode
%  >=2:		start linux
%
/KeyEvent {
  % timeout

  dup 0 eq { boot.buf buildcmdline 2 return } if

  debug 4 ge {
    % print keycode somewhere
    -1 settransparentcolor
    black setcolor
    500 0 moveto dup print "  " print
  } if

  dup 0xffff and /key exch def
  16 shr 0xffff and /keystat exch def

  % Alt + SysRq + O
  keystat 0x0608 and 0x0608 eq {
    key 0x1800 eq {
      poweroff
    } if
  } if

  key

  keymap _undef ne { mapkey } if

  % some special keys
  debug.input

  % put key through normal input queue
  window.input

  pop

  window.action actExit eq {
    /window.action actNothing def
    "" -1 1 return
  } if

  window.action actCloseInfo eq {
    /window.action actNothing def
    "" -1 3 return
  } if

  window.action actPassword eq {
    /window.action actNothing def
    password.buf -1 3 return
  } if

  window.action actStart eq {
    /window.action actNothing def
    boot.buf buildcmdline 2 return
  } if

  boot.buf menu.entry 0
} def


/bc.cmd 256 malloc def

% ( option_string ) ==> ( cmdline menu_entry )
%
% grub:
%   just return
%
% syslinux & lilo:
%   check if the commmand line starts with the current kernel name or
%   "linux"; if not, put the kernel name in front of the command line. (This
%   is to keep compatibility with the old scheme requiring the user to write
%   the kernel name explicitly.)
%
/buildcmdline {
  menu.entry 0 lt { -1 return } if
  menu.entry menu.texts length ge { -1 return } if

  /bc.opts exch def
  /bc.kernel menu.texts menu.entry get def

  grub {

    /bc.addkernel false def

  } {

    /bc.addkernel true def

    [ bc.kernel "linux" ] 0 over length 1 sub 1 exch {
      over exch get

      bc.opts over eq {
        /bc.addkernel false def
      } {
        bc.opts over strstr 1 eq {
           bc.opts over length get ' ' eq {
             /bc.addkernel false def
           } if
        } if
      } ifelse

      pop

      bc.addkernel not { exit } if

    } for

    pop

    % special case: option is identical to label
    bc.kernel "apic" eq { /bc.addkernel true def } if

  } ifelse

  bc.addkernel {
    alt.kernel "" ne { alt.kernel } { bc.kernel } ifelse
    "%s " 256 bc.cmd snprintf
  } {
    bc.cmd 0 0 put
  } ifelse

  cmdline.hidden "" ne {
    cmdline.hidden "%s " 256 bc.cmd length sub bc.cmd bc.cmd length add snprintf
  } if

%  syslinux {
%    xmenu.video 0 get 0 eq {
%      "textmode=1 " 256 bc.cmd length sub bc.cmd bc.cmd length add snprintf
%    } if
%    video.modes.list xmenu.video 0 get get 0 get dup {
%      "vga=0x%x " 256 bc.cmd length sub bc.cmd bc.cmd length add snprintf
%    } {
%      pop
%    } ifelse
%  } if
%
%  do_driverupdate _undef ne {
%    "dud=1 " 256 bc.cmd length sub bc.cmd dup length add snprintf
%  } if

  bc.opts "%s " 256 bc.cmd length sub bc.cmd bc.cmd length add snprintf

  bc.cmd dropspaces

  % bc.cmd 0 0 moveto black setcolor show trace

  bc.cmd menu.entry
} def



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( menu_entries_array cmdline_args_array defaultentry ) == > ( )
/MenuInit {
  bsplash.done
  init

  /menu.dentry exch def
  /menu.args exch def
  /menu.texts exch def

  window.main
  dup window.init
      window.show

%  fadein_logo

  syslinux usernote 0 ne and {
    dvd_popup
  } if

} def


% Setup boot option input field.
%
% ( ) == > ( )
%
/bootoptions.init {
  window.current .color.fg get setcolor
  window.current .ed.font get setfont
  boot.ed edit.hidecursor
  menu.args menu.entry get
  menu.texts menu.entry get
  bootpromptmap
  dup
  boot.ed exch edit.init
  "" ne { boot.ed ' ' edit.input } if
} def


/cmdline.hidden 256 malloc def
/cmdline.shown 256 malloc def

% Split command line into hidden and normal part.
%
% ( cmdline_args menu_text ) == > ( new_cmdline_args )
%
% alt.kernel is set if a different kernel should be used (this feature should
% no longer be needed).
%
/bootpromptmap {
  /alt.kernel "" def

%  syslinux {
%    dup "apic"  eq { pop pop "apic" /alt.kernel "linux" def return } if
%  } if
  pop

  /cmdline exch def

  cmdline "showopts" getoption
  dup "" eq {
    cmdline.shown 0 0 put
    pop
    video.options _undef eq { "" } { video.options xmenu.video 0 get get } ifelse
    dup "" eq { "" } { " " } ifelse exch
    debug.options _undef eq { "" } { debug.options xmenu.debug 0 get get } ifelse
    dup "" eq { "" } { " " } ifelse exch
    acpi.options _undef eq { "" } { acpi.options xmenu.acpi 0 get get } ifelse
    dup "" eq { "" } { " " } ifelse exch
    cmdline
    dup "" eq { "" } { " " } ifelse exch
    "%s%s%s%s%s%s%s%s" 256 cmdline.shown snprintf
    cmdline.hidden 0 0 put
  } {
    "showopts" length add skipspaces
    "%s" 256 cmdline.shown snprintf
    video.options _undef eq { "" } { video.options xmenu.video 0 get get } ifelse
    dup "" eq { "" } { " " } ifelse exch
    debug.options _undef eq { "" } { debug.options xmenu.debug 0 get get } ifelse
    dup "" eq { "" } { " " } ifelse exch
    acpi.options _undef eq { "" } { acpi.options xmenu.acpi 0 get get } ifelse
    dup "" eq { "" } { " " } ifelse exch
    cmdline
    dup "" eq { "" } { " " } ifelse exch
    "%s%s%s%s%s%s%s%s" 256 cmdline.hidden snprintf
    cmdline.hidden "showopts" getoption 0 0 put
  } ifelse

  cmdline.shown dropspaces
  cmdline.hidden dropspaces

  cmdline.shown
} def


% Skip leading non-spaces.
%
% ( string ) ==> ( string )
%
/skipnonspaces {
  { dup 0 get dup 0 ne exch ' ' ne and { 1 add } { exit } ifelse } loop
} def


% Skip leading spaces.
%
% ( string ) ==> ( string )
%
/skipspaces {
  { dup 0 get ' ' eq { 1 add } { exit } ifelse } loop
} def


% Drop spaces at string end.
% Modifies string!
%
% ( string ) ==> ( )
%
/dropspaces {
  dup length
  dup 0 eq {
    pop pop
  } {
    1 sub
    -1 0 {
      over over get ' ' eq { over exch 0 put } { pop exit } ifelse
    } for
    pop
  } ifelse
} def


% Test if string[ofs-1]/string[ofs] is a word boundary.
%
% ( string ofs ) ==> ( true|false )
%
% boundary is either space/non-space or non-space/(space|'=')
%
/iswordboundary {
  dup 0 eq { pop pop true return } if

  add dup 1 sub 0 get exch 0 get

  over ' ' eq over ' ' gt and { pop pop true return } if
  over ' ' gt over dup ' ' eq exch dup '=' eq exch 0 eq or or and { pop pop true return } if

  pop pop false
} def


% Get boot option.
%
% ( cmdline option_name ) ==> ( option_start )
%
/getoption {
  /go.name exch def
  /go.cmdline exch def
  /go.pos 0 def
  {
    go.cmdline go.pos add go.name strstr dup {
       1 sub /go.pos exch def

       go.cmdline go.pos iswordboundary
       go.cmdline go.pos go.name length add iswordboundary
       and {
         go.cmdline go.pos add exit
       } {
         /go.pos go.pos 1 add def
       } ifelse
    } {
      pop "" exit
    } ifelse
  } loop
} def


/redrawmenu {
  menu.visible.entries menu.texts length lt menu.scrollbar and {
    /menu.vsb.height1
      menu.shift
      menu.sb.height mul menu.texts length div
    def

    /menu.vsb.height3
      menu.texts length menu.visible.entries menu.shift add sub
      menu.sb.height mul menu.texts length div
    def

    -1 -1 0 fillmode
    lightgray setcolor
    menu.sb.x menu.sb.y menu.vsb.height1 add moveto
    menu.sb.width menu.sb.height menu.vsb.height1 menu.vsb.height3 add sub
    fillrect

    menu.vsb.height1 0 ne {
      menu.sb.x menu.sb.y moveto
      currentpoint menu.sb.width menu.vsb.height1 image
    } if

    menu.vsb.height3 0 ne {
      menu.sb.x menu.sb.y menu.sb.height menu.vsb.height3 sub add moveto
      currentpoint menu.sb.width menu.vsb.height3 image
    } if

  } if

  menu.text.normal setcolor

  /x menu.start.x def
  /y menu.start.y def

  0 1 menu.visible.entries 1 sub {
   x y moveto currentpoint menu.bar.width menu.bar.height image
   x menu.text.xofs add y menu.text.yofs add moveto
   menu.texts exch menu.shift add get menuitemmap
   currentfont exch font2 setfont show setfont
   /y y menu.item.height add def
  } for

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( entry status ) ==> ( )
%
% status:
%  false	not selected
%  true		selected
%
/MenuSelect {
  /menu.status exch def

  /menu.idx over def

  menu.shift sub
  menu.item.height mul menu.start.y add
  menu.start.x exch
  moveto

  -1 -1 0 fillmode

  currentpoint
  menu.status {
    menu.bar.color
    setcolor menu.bar.width menu.bar.height fillrect
  } {
    currentpoint menu.bar.width menu.bar.height image
  } ifelse
  moveto

  menu.text.xofs menu.text.yofs rmoveto
  menu.status {
    menu.text.select
  } {
    menu.text.normal
  } ifelse
  setcolor
  menu.texts menu.idx get menuitemmap
  currentfont exch font2 setfont show setfont

  menu.status {
    % init boot options
    bootoptions.init

    % set help context
    "main" help.setcontext
    menu.texts menu.idx get
    dup help.findpage "" eq {
      pop
    } {
      help.setcontext
    } ifelse
  } if

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( text errorcode ) ==> ( )
% errorcode:
%   0	normal info
%   1	fatal error
%   2	missing kernel
%

/InfoBoxInit {
  /info.type exch def
  /info.msg exch def

  window.dialog

  info.type 1 eq {
    dup .title.bg red put
    dup .title "I/O hiba" put
    dup .buttons
    [
      button.reboot button.default actCloseInfo button.setaction
    ] put
  } {
    dup .title "Rendszerbetlt" put
    dup .buttons
    [
      button.ok button.default actCloseInfo button.setaction
%      button.cancel button.notdefault actCloseInfo button.setaction
    ] put
  } ifelse

  syslinux
  info.type 2 eq and
  usernote 0 ne and {
    "\n\nMaybe you booted from the 2nd side of the DVD.\nIf so, turn the DVD over and try again." info.msg
    over length info.msg length 1 add add "%s%s" exch
    dup malloc dup /info.msg exch def snprintf
  } if

  dup .text info.msg put

  dup window.init
      window.show

} def



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

% progress bar code


% Show percentage of progress bar.
%
% ( percentage ) ==> ( )
%
/progress.percent {
  0 max 100 min		% so people don't ask silly questions...
  "100%" strsize over neg progress.text.x add progress.text.y moveto
  window.current .color.bg get setcolor
  fillrect  

  "%3u%%" 8 dup malloc dup 5 1 roll snprintf

  dup strsize pop neg progress.text.x add progress.text.y moveto
  window.current .color.fg get setcolor
  dup show
  free

} def


% Show n-th progress bar symbol.
%
% ( n ) ==> ( )
%
/progress.sym.show {
  /progress.sym.current exch def

  progress.bar.x progress.bar.y moveto
  progress.sym.width progress.sym.current 1 sub mul 1 add 1 rmoveto
  progress.sym.width 2 sub
  progress.bar.height 2 sub
  loading_color setcolor
  fillrect

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( kernel_name ) ==> ( )
/ProgressInit {
  /progress.kname exch def

  boot.ed edit.hidecursor

  /dia window.dialog def

  dia .width.min 330 put
  dia .position 10 put

  dia .title "Indts..." put
  dia .text
    progress.kname "memtest" eq {
      "Memria ellenrz betltse\n\n\n"
    } {
      "A rendszer betltse\n\n\n"
    } ifelse
  put

  dia window.init
  dia window.show

  % now add progress bar

  dia .x get dia .y get moveto
  dia .text.x get dia .text.y get 28 add rmoveto

  /progress.bar.height 19 def
  /progress.bar.width dia .width get 60 sub def

  /progress.sym.width 10 def
  /progress.bar.width
    progress.bar.width progress.sym.width div
    /progress.syms over def progress.sym.width mul
  def

  currentpoint over 1 sub over 2 sub moveto
  black white progress.bar.width 2 add progress.bar.height 4 add drawborder

  /progress.bar.y exch def
  /progress.bar.x exch def

  /progress.text.x progress.bar.x progress.bar.width 37 add add def
  /progress.text.y progress.bar.y progress.bar.height fontsize exch pop sub 2 div add def

  /progress.sym.current 0 def

  0 progress.percent

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( ) ==> ( )
/ProgressDone {
  window.done
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( max current ) ==> ( )
/ProgressUpdate {
  over over 100 mul exch 1 max div progress.percent

  progress.syms mul progress.syms 2 div add exch 1 max div

  0 max progress.syms min

  dup progress.sym.current gt {
    progress.sym.current 1 add over 1 exch {
      progress.sym.show
    } for
  } if
  pop

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( timeout time ) ==> ( )
/Timeout {
  % first time
  timeout.time _undef eq { timeout.init } if

  dup /timeout.time exch def

  over sub neg timeout.symbols mul exch div

  dup timeout.current eq { pop return } if

  /timeout.last timeout.current def
  /timeout.current exch def

  timeout.current timeout.symbols ge {
    timeout.x timeout.time.y moveto
    currentpoint timeout.dx timeout.symbols mul fontsize exch pop image
    0 1 timeout.symbols 1 sub {
      timeout.clear { 2 } { 1 } ifelse drawtsymbol
    } for
  } {
    timeout.time.x timeout.time.y moveto
    currentpoint currentpoint 200 fontsize exch pop image
    moveto
    black setcolor
    timeout.time 1080 mul 16000 add 19663 div "%d msodpercen bell" 64 timeout.buf snprintf timeout.buf show
    timeout.last 1 timeout.current {
      1 sub dup 0 ge {
        1 drawtsymbol
      } if
    } for
  } ifelse
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% draw a timeout symbol
% ( index status ) ==> ()
%
% status: 0: init, 1: clear, 2: set
%
/drawtsymbol {
  /timeout.status exch def
  dup timeout.dx mul timeout.x add exch
  timeout.dy mul timeout.y add
  moveto currentpoint
  timeout.status 2 eq {
    currentpoint timeout.size image
  } {
    timeout.status 0 eq {
      pop pop
    } {
      0 462 timeout.size image
    } ifelse
  } ifelse
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( time ) ==> ( )
% /Timer { pop } def
%/Timer {
%  1080 mul 19663 div
%  dup prevsec eq {
%    pop
%  } {
%  /prevsec over def
%  dup 60 mod /second exch def 60 div dup 60 mod /minute exch def 60 div /hour exch def
%    currentfont
%    font2 setfont
%    currentpoint
%    currentcolor
%    clockwidth _undef eq {
%      "00:00:00" strsize /clockheight exch def /clockwidth exch def
%      /clockposx 2 def
%      /clockposy 2 def
%      clockposx clockposy moveto
%      /clockarea clockwidth clockheight savescreen def
%    } {
%      clockposx clockposy moveto
%      clockarea restorescreen
%    } ifelse
%    clockposx clockposy moveto
%    black setcolor
%    timestring _undef eq { /timestring 9 malloc def } if
%    second minute hour "%02d:%02d:%02d" 9 timestring snprintf
%    timestring show
%    setcolor
%    moveto
%    setfont
%  } ifelse
%} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( label correct_password ) ==> ( )
%
/PasswordInit {
  /password.key exch def pop

  /dia window.dialog def

  dia .title "Password" put
  dia .text "Please enter your password:   \n\n\n" put
  dia .buttons
    [ button.ok button.default actPassword button.setaction ]
  put

  dia window.init
  dia window.show

  % add input field

  % show only '*'
  dia .ed.font dia .font get 0x80 or put

  dia .x get dia .y get moveto
  dia .text.x get dia .text.y get 30 add rmoveto

  /password.ed.width dia .width get 20 sub def
  /password.ed.height dia .ed.font get setfont fontsize exch pop 2 add def

  currentpoint over 1 sub over 2 sub moveto
  black white password.ed.width password.ed.height 4 add drawborder
  moveto

  black setcolor

  dia .ed [
    currentpoint
    password.ed.width password.ed.height savescreen
    password.buf
    password.buf.size
    0
    0
    0
  ] put

  dia .ed get "" edit.init
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( password ) ==> ( error )
%
% error:
%   true   password ok
%   false  wrong password
%
% ****** FIXME: test result seems to be unused
%
/PasswordDone {

  password.key eq
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( text ) == > ( new_text )
/menuitemmap {
  dup "memtest" eq over "memtest86" eq or { pop "Memria ellenrzse" return } if
  syslinux {
    dup "install"   eq { pop "Telepts"                        return } if
    dup "harddisk"  eq { pop "Teleptett rendszer indtsa"     return } if
    dup "rescue"    eq { pop "Karbantart rendszer"             return } if
    dup "server"    eq { pop "Kiszolgl hlzati teleptshez" return } if
  } {
    dup "linux"     eq { pop "Linux"                 return } if
    dup "failsafe"  eq { pop "Linux - Safe Settings" return } if
    dup "windows"   eq { pop "Windows"               return } if
  } ifelse
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( color0 color1 width height ) ==> ( )
/drawborder {
  currentpoint /db.y0 exch def /db.x0 exch def

  /db.y1 exch 1 sub db.y0 add def
  /db.x1 exch 1 sub db.x0 add def
  /db.col1 exch def
  /db.col0 exch def

  db.x0 db.y1 moveto

  db.col0 setcolor
  db.x0 db.y0 lineto db.x1 db.y0 lineto

  db.col1 setcolor
  db.x1 db.y1 lineto db.x0 db.y1 lineto
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% ( color0 color1 color2 width height ) ==> ( )
% draw frame with shadow
% color0: upper left, color1: lower right, color2: shadow
/drawborder3 {
  currentpoint /db.y0 exch def /db.x0 exch def

  /db.y1 exch 1 sub db.y0 add def
  /db.x1 exch 1 sub db.x0 add def
  /db.col2 exch def
  /db.col1 exch def
  /db.col0 exch def

  db.x0 db.y1 moveto

  db.col0 setcolor
  db.x0 db.y0 lineto db.x1 db.y0 lineto

  db.col1 setcolor
  db.x1 db.y1 lineto db.x0 db.y1 lineto

  db.col2 -1 ne {
    db.col2 setcolor
    1 1 rmoveto
    db.x1 1 add db.y1 1 add lineto
    db.x1 1 add db.y0 1 add lineto
  } if
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% center text
% ( text width height ) ==> ( )
/centertext {
  3 -1 roll strsize
  4 2 roll
  4 1 roll exch 4 1 roll sub 2 div neg 3 1 roll sub 2 div neg
} def


/fadeout_logo {
  /cols 256 malloc def

  0 1 255 { cols exch 0 put } for

  10 1 150 {
    10 1 100 {
       over exch moveto getpixel
       cols exch 1 put
    } for
    pop
  } for
  cols 10 10 moveto getpixel 0 fade
} def

/fadein_logo {
  cols 10 10 moveto getpixel fadein
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

% some global vars


% Allocate and define a new color.
%
% ( palette ) ==> ( color )
%
/newcolor {
  max_image_colors newcolor.count add
  dup rot setpalette
  /newcolor.count newcolor.count 1 add def
  def
} def


/max_image_colors image.colors def

/font1 font_9x15 1 add def
/font2 font1 def
/font3 font_10x20 1 add def

/newcolor.count 0 def

/bblack               0x000000          newcolor
/bred                 0xc00000          newcolor

bsplash.show

/black               0x000000          newcolor
/white               0xffffff          newcolor
/mediumblue          0x566eb2          newcolor
/mediumblue2         0x768cc5          newcolor
/lightblue           0xd0d0ff          newcolor
% /lightblue2          0x5c91ea          newcolor
/lightblue2          0x2a73d3          newcolor
/darkgray            0x4c4c4c          newcolor
/grayb5              0xb5b5b5          newcolor
% /lightgray           0xeeeaee          newcolor
/lightgray           0xececf4          newcolor
/graye8              0xe8e8e8          newcolor
/blue                0x0000a0          newcolor
/yellow              0xffff20          newcolor
/red                 0xc00000          newcolor
/somered             0xe6373a          newcolor
/loading_color       0x23449c          newcolor
% /loading_color       0x2f8e17          newcolor
% /loading_color       0x248617          newcolor
/green               0x009000          newcolor
/fn_color            0x33449c          newcolor
% /fn_color            0x8abe5a          newcolor
% /fn_color            0x6fb530          newcolor
/fn_color2           0x53a53f          newcolor
/prog_frame_color    lightblue def
% /prog_frame_color3   0x3b7f10          newcolor
/prog_frame_color2   0x364c65          newcolor
/dud_color           0x006f64          newcolor

/init {
  0 0 moveto getpixel setcolor
  0 0 moveto screen.size fillrect
  loadpalette
  0 0 moveto currentpoint screen.size image

  font2 setfont

  /menu.entry          -1 def
  /menu.text.xofs      10 def
  /menu.text.yofs       2 def
  /menu.item.height    fontsize exch pop dup 2 div add def
  /menu.bar.height fontsize exch pop menu.text.yofs dup add add def

  font1 setfont

  /menu.text.normal    black             def
  /menu.text.select    white             def
  /boot.text.options   black             def
  /boot.text.normal    black             def
  /infobox.bg          lightgray         def
  /infobox.text.normal black             def
  /frame.darkcolor     grayb5            def
  /frame.lightcolor    graye8            def
  /frame.shadowcolor   mediumblue          def
%  /frame.shadowcolor   -1          def

  /menu.bar.color      fn_color       def

  /frame1.pos  { 160 100 } def
  /frame1.size { 320 195 } def
  /frame2.pos  {  10 410 } def
  /frame2.size { 620  25 } def
  /frame3.pos  {  10 386 } def
  /frame3.size { 124  25 } def
  /frame4.pos  { 560  62 } def
  /frame4.size {  22 264 } def

  /menu.start.x frame1.pos pop 10 add def
  /menu.start.y frame1.pos exch pop 12 add def

  /menu.bar.width frame1.size pop 20 sub def
  /menu.max.entries 8 def
  /menu.scrollbar false def

  /boot.buf.size 256 def
  /boot.buf boot.buf.size malloc def

  /ms.size     {  22  22 } def
  /ms.up       {   0 480 }  def
  /ms.down     {  23 480 }  def

  % password buffer
  /password.buf.size 32 def
  /password.buf password.buf.size malloc def

  /update.text "Please get your Driver Update Floppy ready." def
  /update.pos { 260 383 } def

%  frame1.pos moveto lightgray setcolor frame1.size fillrect
  frame1.pos moveto prog_frame_color dup prog_frame_color2 frame1.size drawborder3

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Some special debug & test keys.
%
% ( key_in ) ==> ( key_out )
%
/debug.input {
  dup keyF5 eq syslinux and {
    0 kroete idle
    pop 0
  } if

  dup keyF6 eq syslinux and {
    1 kroete idle
    pop 0
  } if

  dup keyShiftF3 eq syslinux and debug 3 ge and {
    currentcolor black setcolor
    currentpoint 0 0 moveto
    "eject  " print bootdrive eject print
    moveto setcolor
    pop 0
  } if

  dup keyShiftF5 eq syslinux and debug 3 ge and {
    currentcolor black setcolor
    currentpoint 100 0 moveto
    bootdrive print
    moveto setcolor
    pop 0
  } if

  dup keyShiftF7 eq syslinux and {
    0 -1 idle
    pop 0
  } if

  dup keyShiftF8 eq debug 3 ge and {
    currentcolor black setcolor currentpoint 300 0 moveto
    memsize print "/" print print "  " print
    moveto setcolor
    pop 0
  } if

  dup keyShiftF9 eq debug 3 ge and {
    pop 0
  } if

  dup keyShiftF10 eq {
    /debug debug 1 add def
    debug 2 eq {
      0 0 moveto
      syslinux {
        "syslinux" show
      } {
        lilo {
          "lilo" show
        } {
          grub {
            "grub" show
          } {
            "unknown" show
          } ifelse
        } ifelse
      } ifelse
    } if
    pop 0
  } if
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Show exit popup.
%
% ( ) ==> ( )
%
/exit_popup {
  window.dialog

  dup .title "Viszlt..." put
  dup .text "Biztosan ki akar lpni szveges mdba?\nFIGYELEM: Ez csak gyakorlott felhasznlknak ajnlott!" put
  dup .buttons [
    button.ok button.default actExit button.setaction
    button.cancel button.notdefault actNothing button.setaction
  ] put
  dup window.init
      window.show

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Show help window.
%
% ( ) ==> ( )
%
/show_help {
  window.help
  
  dup window.init
  window.show

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Show dvd popup.
%
% ( ) ==> ( )
%
/dvd_popup {
  window.dialog

  dup .title "You're almost there..." put
  dup .text "This is a two-sided DVD. You have booted from the 2nd side.\n\nPlease turn the DVD over, then continue." put
  dup .buttons [
%    button.eject    button.default    actEject actNoClose or button.setaction
    button.continue button.default actNothing button.setaction
  ] put
  dup window.init
      window.show

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Show "power off" popup.
%
% ( ) ==> ( )
%
/power_off {
  window.dialog

  dup .title "Kikapcsols" put
  dup .text "Biztosan ki akarja kapcsolni a szmtgpet?" put
  dup .buttons [
    button.ok     button.notdefault    actPowerOff actNoClose or button.setaction
    button.cancel button.default       actNothing                button.setaction
  ] put
  dup window.init
      window.show

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Initialize timeout indicator.
%
% ( ) ==> ( )
%
/timeout.init {
  /timeout.current    -1 def
  /timeout.x         160 def
  /timeout.y         340 def
  /timeout.dx          1 def
  /timeout.dy          0 def
  /timeout.symbols   320 def
  /timeout.clear    true def
  /timeout.size      { 1 18 } def
  /timeout.buf        64 malloc def

  timeout.x timeout.y moveto
  prog_frame_color dup
  timeout.dx timeout.symbols mul timeout.size exch pop
  drawborder

  timeout.x timeout.y 20 sub moveto
  black setcolor
  "Indts " show
  currentpoint /timeout.time.y exch def /timeout.time.x exch def
} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Search for option in cmdline.
% Returns _undef if not found.
%
% ( cmdline option_name ) ==> ( option_start )
%
/bootopt.find {
  /bo.opt exch def
  /bo.cmdline exch def

  {
    bo.cmdline bo.opt strstr
    dup {
      dup 1 eq {
        true
      } {
        dup 2 sub bo.cmdline exch get ' ' eq
      } ifelse

      {
        bo.cmdline over bo.opt length add 1 sub get
        dup '=' eq
        over ' ' eq or
        exch 0 eq or
      } {
        false
      } ifelse

      bo.cmdline rot add exch

      {
        1 sub exit
      } {
        /bo.cmdline exch def
      } ifelse
    } {
      pop
      _undef exit
    } ifelse
  } loop

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Copy src to dst.
%
% ( dst src ) ==> ( )
%
/strcpy {
  {
    over over 0 get dup 0 exch
    {
      exch put
      1 add exch 1 add exch
    } {
      exch put
      exit
    } ifelse
  } loop

  pop pop
} def

%% include window.inc
%% include button.inc
%% include help.inc
%% include main.inc
%% include xmenu.inc
%% include keytables.inc

